# ISAAC_SET_PREFIX(PREFIX)
#---------------------------
AC_DEFUN([ISAAC_SET_PREFIX],
[
    unset CDPATH
    # make $PIPE_HOME the default for the installation
    AC_PREFIX_DEFAULT($1)

    if test "x$prefix" = "xNONE"; then
        prefix=$ac_default_prefix
        ac_configure_args="$ac_configure_args --prefix $prefix"
    fi

    if test "x$exec_prefix" = "xNONE"; then
        exec_prefix=$prefix
    fi

])


# ISAAC_SET_VERSION_INFO(VERSION, [CURRENT], [REVISION], [AGE])
#----------------------------------------------------------------
# Setup various version information, especially the libtool versioning
AC_DEFUN([ISAAC_SET_VERSION_INFO],
[
    isaac_version=`echo "$1" | sed -e 's/[[a-z,A-Z]].*$//'`

    isaac_major_version=`echo "$isaac_version" | \
        sed 's/\([[0-9]]*\).\(.*\)/\1/'`
    isaac_minor_version=`echo "$isaac_version" | \
        sed 's/\([[0-9]]*\).\([[0-9]]*\)\(.*\)/\2/'`
    isaac_micro_version=`echo "$isaac_version" | \
        sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`

    if test -z "$isaac_major_version"; then isaac_major_version=0
    fi

    if test -z "$isaac_minor_version"; then isaac_minor_version=0
    fi

    if test -z "$isaac_micro_version"; then isaac_micro_version=0
    fi

    ISAAC_VERSION="$isaac_version"
    ISAAC_MAJOR_VERSION=$isaac_major_version
    ISAAC_MINOR_VERSION=$isaac_minor_version
    ISAAC_MICRO_VERSION=$isaac_micro_version

    if test -z "$4"; then ISAAC_INTERFACE_AGE=0
    else ISAAC_INTERFACE_AGE="$4"
    fi

    ISAAC_BINARY_AGE=`expr 100 '*' $ISAAC_MINOR_VERSION + $ISAAC_MICRO_VERSION`
    ISAAC_BINARY_VERSION=`expr 10000 '*' $ISAAC_MAJOR_VERSION + \
                          $ISAAC_BINARY_AGE`

    AC_SUBST(ISAAC_VERSION)
    AC_SUBST(ISAAC_MAJOR_VERSION)
    AC_SUBST(ISAAC_MINOR_VERSION)
    AC_SUBST(ISAAC_MICRO_VERSION)
    AC_SUBST(ISAAC_INTERFACE_AGE)
    AC_SUBST(ISAAC_BINARY_VERSION)
    AC_SUBST(ISAAC_BINARY_AGE)

    AC_DEFINE_UNQUOTED(ISAAC_MAJOR_VERSION, $ISAAC_MAJOR_VERSION,
                       [ISAAC major version number])
    AC_DEFINE_UNQUOTED(ISAAC_MINOR_VERSION, $ISAAC_MINOR_VERSION,
                       [ISAAC minor version number])
    AC_DEFINE_UNQUOTED(ISAAC_MICRO_VERSION, $ISAAC_MICRO_VERSION,
                       [ISAAC micro version number])
    AC_DEFINE_UNQUOTED(ISAAC_INTERFACE_AGE, $ISAAC_INTERFACE_AGE,
                       [ISAAC interface age])
    AC_DEFINE_UNQUOTED(ISAAC_BINARY_VERSION, $ISAAC_BINARY_VERSION,
                       [ISAAC binary version number])
    AC_DEFINE_UNQUOTED(ISAAC_BINARY_AGE, $ISAAC_BINARY_AGE,
                       [ISAAC binary age])

    ESO_SET_LIBRARY_VERSION([$2], [$3], [$4])
])


# ISAAC_SET_PATHS
#------------------
# Define auxiliary directories of the installed directory tree.
AC_DEFUN([ISAAC_SET_PATHS],
[

    if test -z "$plugindir"; then
        plugindir='${libdir}/esopipes-plugins/${PACKAGE}-${VERSION}'
    fi

    if test -z "$privatelibdir"; then
        privatelibdir='${libdir}/${PACKAGE}-${VERSION}'
    fi

    if test -z "$apidocdir"; then
        apidocdir='${datadir}/doc/esopipes/${PACKAGE}-${VERSION}/html'
    fi

    if test -z "$pipedocsdir"; then
        pipedocsdir='${datadir}/doc/esopipes/${PACKAGE}-${VERSION}'
    fi

    if test -z "$configdir"; then
        configdir='${datadir}/esopipes/${PACKAGE}-${VERSION}/config'
    fi

    AC_SUBST(plugindir)
    AC_SUBST(privatelibdir)
    AC_SUBST(apidocdir)
    AC_SUBST(pipedocsdir)
    AC_SUBST(configdir)

    # Define a preprocesor symbol for the plugin search paths

    AC_DEFINE_UNQUOTED(ISAAC_PLUGIN_DIR, "esopipes-plugins",
                       [Plugin directory tree prefix])

    eval plugin_dir="$plugindir"
    plugin_path=`eval echo $plugin_dir | \
                sed -e "s/\/${PACKAGE}-${VERSION}.*$//"`

    AC_DEFINE_UNQUOTED(ISAAC_PLUGIN_PATH, "$plugin_path",
                       [Absolute path to the plugin directory tree])

])


# ISAAC_CREATE_SYMBOLS
#-----------------------
# Define include and library related makefile symbols
AC_DEFUN([ISAAC_CREATE_SYMBOLS],
[

    # Symbols for package include file and library search paths

    ISAAC_INCLUDES='-I$(top_srcdir)/isaac -I$(top_srcdir)/irplib'
    ISAAC_LDFLAGS='-L$(top_builddir)/isaac'

    all_includes='$(ISAAC_INCLUDES) $(CPL_INCLUDES) $(EXTRA_INCLUDES)'
    all_ldflags='$(ISAAC_LDFLAGS) $(CPL_LDFLAGS) $(EXTRA_LDFLAGS)'

    # Library aliases

    LIBISAAC='$(top_builddir)/isaac/libisaac.la'
    LIBIRPLIB='$(top_builddir)/irplib/libirplib.la'

    # Substitute the defined symbols

    AC_SUBST(ISAAC_INCLUDES)
    AC_SUBST(ISAAC_LDFLAGS)

    AC_SUBST(LIBISAAC)
    AC_SUBST(LIBIRPLIB)

    # Check for CPL and user defined libraries
    AC_REQUIRE([CPL_CHECK_LIBS])
    AC_REQUIRE([ESO_CHECK_EXTRA_LIBS])

    all_includes='$(ISAAC_INCLUDES) $(CPL_INCLUDES) $(EXTRA_INCLUDES)'
    all_ldflags='$(ISAAC_LDFLAGS) $(CPL_LDFLAGS) $(EXTRA_LDFLAGS)'

    AC_SUBST(all_includes)
    AC_SUBST(all_ldflags)
])


# ISAAC_ENABLE_STRICT(strict=no)
#-----------------------------
AC_DEFUN([ISAAC_ENABLE_STRICT],
[
    AC_REQUIRE([AC_PROG_EGREP])
    AC_REQUIRE([AC_PROG_CC])

    AC_ARG_ENABLE(strict,
                  AS_HELP_STRING([--enable-strict],
                                 [compiles with extra, strict compiler options (may not work!) [default=$1]]),
                  isaac_enable_strict=$enableval, isaac_enable_strict=$1)

    AC_CACHE_CHECK([whether extra, strict compiler options should be used],
                   isaac_cv_enable_strict,
                   isaac_cv_enable_strict=$isaac_enable_strict)


    if test x"$isaac_cv_enable_strict" = xyes; then
    
        isaac_enable_strict_std_set=no

        if test -n "$CFLAGS"; then
            echo $CFLAGS | $EGREP '(\-std=|-ansi)' >/dev/null 2>&1
            if test x"$?" = x0; then
                isaac_enable_strict_std_set=yes
            fi
        fi
        
        if test x"$isaac_enable_strict_std_set" = xno; then
                ESO_PROG_CC_FLAG([std=c99], [CFLAGS="$CFLAGS -std=c99"])
        fi
        
        ESO_PROG_CC_FLAG([pedantic], [CFLAGS="$CFLAGS -pedantic"])

        ESO_PROG_CC_FLAG([Werror=implicit-function-declaration],
                         [CFLAGS="$CFLAGS -Werror=implicit-function-declaration"])
        ESO_PROG_CC_FLAG([Werror=incompatible-pointer-types],
                         [CFLAGS="$CFLAGS -Werror=incompatible-pointer-types"])
        ESO_PROG_CC_FLAG([Werror=format=2], [CFLAGS="$CFLAGS -Werror=format=2"])
        ESO_PROG_CC_FLAG([Werror=format-nonliteral],
                         [CFLAGS="$CFLAGS -Werror=format-nonliteral"])
        ESO_PROG_CC_FLAG([Werror=format-security],
                         [CFLAGS=" $CFLAGS -Werror=format-security"])
        ESO_PROG_CC_FLAG([Werror=format-y2k],
                         [CFLAGS="$CFLAGS -Werror=format-y2k"])
        ESO_PROG_CC_FLAG([Werror=init-self],
                         [CFLAGS="$CFLAGS -Werror=init-self"])
        ESO_PROG_CC_FLAG([Werror=missing-declarations],
                         [CFLAGS="$CFLAGS -Werror=missing-declarations"])
        ESO_PROG_CC_FLAG([Werror=missing-format-attribute],
                         [CFLAGS="$CFLAGS -Werror=missing-format-attribute"])
        ESO_PROG_CC_FLAG([Werror=missing-noreturn],
                         [CFLAGS="$CFLAGS -Werror=missing-noreturn"])
        ESO_PROG_CC_FLAG([Werror=nested-externs],
                         [CFLAGS="$CFLAGS -Werror=nested-externs"])
        ESO_PROG_CC_FLAG([Werror=pointer-arith],
                         [CFLAGS="$CFLAGS -Werror=pointer-arith"])
        ESO_PROG_CC_FLAG([Werror=sign-compare],
                         [CFLAGS="$CFLAGS -Werror=sign-compare"])
        ESO_PROG_CC_FLAG([Werror=undef],
                         [CFLAGS="$CFLAGS -Werror=undef"])
        ESO_PROG_CC_FLAG([Werror=variadic-macros],
                         [CFLAGS="$CFLAGS -Werror=variadic-macros"])
        ESO_PROG_CC_FLAG([Werror=volatile-register-var],
                         [CFLAGS="$CFLAGS -Werror=volatile-register-var"])
        ESO_PROG_CC_FLAG([Werror=write-strings],
                         [CFLAGS="$CFLAGS -Werror=write-strings"])
        ESO_PROG_CC_FLAG([Werror=old-style-definition],
                         [CFLAGS="$CFLAGS -Werror=old-style-definition"])
        ESO_PROG_CC_FLAG([Werror=missing-prototypes],
                         [CFLAGS="$CFLAGS -Werror=missing-prototypes"])
        ESO_PROG_CC_FLAG([Werror=maybe-uninitialized],
                         [CFLAGS="$CFLAGS -Werror=maybe-uninitialized"])
        ESO_PROG_CC_FLAG([Werror=discarded-qualifiers],
                         [CFLAGS="$CFLAGS -Werror=discarded-qualifiers"])
        ESO_PROG_CC_FLAG([Werror=unused-but-set-variable],
                         [CFLAGS="$CFLAGS -Werror=unused-but-set-variable"])
        ESO_PROG_CC_FLAG([Werror=unused-parameter],
                         [CFLAGS="$CFLAGS -Werror=unused-parameter"])
        ESO_PROG_CC_FLAG([Werror=unused-variable],
                         [CFLAGS="$CFLAGS -Werror=unused-variable"])

        # Cannot make this an error due to include <unistd.h> on Mac OSX 10.8
        ESO_PROG_CC_FLAG([Wredundant-decls],
                         [CFLAGS="$CFLAGS -Wredundant-decls"])
        # Cannot make this an error w. fno-common on Mac OSX 10.9
        ESO_PROG_CC_FLAG([Wunused-function],
                         [CFLAGS="$CFLAGS -Wunused-function"])

        ESO_PROG_CC_FLAG([fno-common], [CFLAGS="$CFLAGS -fno-common"])
        ESO_PROG_CC_FLAG([fstrict-aliasing], [CFLAGS="$CFLAGS -fstrict-aliasing"])
        
    fi
])
